/*  Fan cart driver version 2.1
    Eric Ayars
    5/24/11
    Modification of previous 'fancart2.pde' to allow use on attiny84.
    
    This program best viewed with 4-space tab stops.
*/

// This can be compiled for either an Arduino or an ATtiny84-based board.
// For running on the ATtiny84, uncomment the following definition:
//#define ATTINY84 
// The compiler sets the pins according to whether ATTINY84 is defined.

#ifdef ATTINY84             // pins for the ATtiny84-based board
    #define MOTORPWM 2
    #define MOTORDIR 3
    #define POWERINDICATOR 7
    #define MODESWITCH 6
    #define SPEEDSWITCH 4
    #define TIMESWITCH 5
    #define MAGNETSWITCH 1
    #define MODELIGHT 8
    #define SPEEDLIGHT 9
    #define TIMELIGHT 0
#else                       // pins for the Arduino-based board 
    #define MOTORPWM 11
    #define MOTORDIR 12
    #define POWERINDICATOR 13
    #define MODESWITCH 2
    #define SPEEDSWITCH 5
    #define TIMESWITCH 7
    #define MAGNETSWITCH 8
    #define MODELIGHT 3
    #define SPEEDLIGHT 4
    #define TIMELIGHT 6
#endif

#define DEBOUNCE 100        // these time intervals work well for me, but
#define FLASHTIME 150       // change them if the user interface isn't
#define UIDELAY 500         // optimal for you.

byte UserInput = 0;
boolean ProgramState = false;
long int CurrentTime;
byte Speed = 2; // Ranges from 1 to 3 (0 not used).
byte PowerLevel[] = {0, 130, 200, 255};
byte Mode = 1;  // Ranges from 1 to 3 currently. Change for more modes.
byte Time = 2;  // Ranges from 1 to 4 seconds. (0 not used).
int TimeLevel[] = {0, 1000, 2000, 3000, 4000};
boolean MotorState = false;
boolean MotorDirection = false; // False is the normal 'forward' dirction.
byte LastMagnetState = HIGH;

byte ButtonDown() {
    // Returns 3-bit indicator of which buttons are currently down.
    // bit 0 - Mode
    // bit 1 - Speed
    // bit 2 - Time
    byte result = 0;    // Default case -- nothing is pressed

    if (digitalRead(MODESWITCH) == LOW) {
        // Mode button is down
        result = result | B0001;
    }

    if (digitalRead(SPEEDSWITCH) == LOW) {
        // Speed button is down
        result = result | B0010;
    }

    if (digitalRead(TIMESWITCH) == LOW) {
        // Time button is down
        result = result | B0100;
    }

    delay(DEBOUNCE);
    return result;
}
    
boolean MagnetSensed() {
    // This returns true or false depending on whether the cart just
    // passed a magnet.
    boolean state;
    byte CurrentState = digitalRead(MAGNETSWITCH);
    if ((LastMagnetState == HIGH) && (CurrentState == LOW)) {
        // Cart has just passed a magnet
        state = true;
        delay(DEBOUNCE);
    } else {
        state = false;
    }
    LastMagnetState = CurrentState;
    return state;
}

void AnnounceReady() {
    // Turns on all three lights to indicate it's ready to go.
    digitalWrite(MODELIGHT, HIGH);
    digitalWrite(SPEEDLIGHT, HIGH);
    digitalWrite(TIMELIGHT, HIGH);
}

void ShowStatus(byte Light, byte Blinks) {
    // Blinks a given light some number of times.
    // Used to show how things are set.
    digitalWrite(Light, LOW);
    delay(FLASHTIME);
    for (byte j=0; j<Blinks; j++) {
        digitalWrite(Light, HIGH);
        delay(FLASHTIME);
        digitalWrite(Light, LOW);
        delay(FLASHTIME);
    }
    delay(UIDELAY);
}

void MotorOn(byte howFast) {
    // turns motor on in direction set by global MotorDirection.
    digitalWrite(POWERINDICATOR, HIGH);
    digitalWrite(MOTORDIR, MotorDirection);
    if (MotorDirection) {
        // set pwm appropriately for backward spin
        analogWrite(MOTORPWM, 255-howFast);
    } else {
        // set pwm for forward spin
        analogWrite(MOTORPWM, howFast);
    }
    MotorState = true;
}

void MotorOff() {
    // turns motor off.
    digitalWrite(POWERINDICATOR, LOW);
    digitalWrite(MOTORDIR, LOW);
    digitalWrite(MOTORPWM, LOW);
    MotorState = false;
}

void ProgramMode(byte ProgramThis) {
    // Sets mode, speed, time...
    boolean State = false;
    boolean Exit = false;
    
    if (ProgramThis & B0001) {  // Program the Mode.
        // Turn mode light off and the other two lights on.
        digitalWrite(MODELIGHT, LOW);
        digitalWrite(SPEEDLIGHT, HIGH);
        digitalWrite(TIMELIGHT, HIGH);
        
        while (!Exit) {
            delay(DEBOUNCE);
            if (ButtonDown() == 1) {
                // Mode button has been pressed. Increment mode.
                Mode += 1;
                if (Mode > 3) {
                    Mode = 1;
                }
                ShowStatus(MODELIGHT, Mode);
            }
            UserInput = ButtonDown();
            if (UserInput > 1) {
                Exit = true;
            }
        }
    }

    if (ProgramThis & B0010) {  // Program the Speed.
        // Turn speed light off and the other two lights on.
        digitalWrite(MODELIGHT, HIGH);
        digitalWrite(SPEEDLIGHT, LOW);
        digitalWrite(TIMELIGHT, HIGH);
        
        while (!Exit) {
            delay(DEBOUNCE);
            if (ButtonDown() == 2) {
                // Speed button has been pressed. Increment Speed.
                Speed += 1;
                if (Speed > 3) {
                    Speed = 1;
                }
                ShowStatus(SPEEDLIGHT, Speed);
            }
            UserInput = ButtonDown();
            if ((UserInput & B0001) || (UserInput & B0100)) {
                Exit = true;
            }
        }
    }

    if (ProgramThis & B0100) {  // Program the Time.
        // Turn Time light off and the other two lights on.
        digitalWrite(MODELIGHT, HIGH);
        digitalWrite(SPEEDLIGHT, HIGH);
        digitalWrite(TIMELIGHT, LOW);
        
        while (!Exit) {
            delay(DEBOUNCE);
            if (ButtonDown() == 4) {
                // Time button has been pressed. Increment Time.
                Time += 1;
                if (Time > 4) {
                    Time = 1;
                }
                ShowStatus(TIMELIGHT, Time);
            }
            UserInput = ButtonDown();
            if ((UserInput & B0001) || (UserInput & B0010)) {
                Exit = true;
            }
        }
    }
}

void DoMode(byte Mode) {
    // This routine runs the desired mode until a button is pressed.

    AnnounceReady();
    MotorDirection = false; // false is forward, true is backwards.

    while (!ButtonDown()) {
        // No button has been pressed, so keep going.

        // Mode 1
        //
        // In mode 1, the fan turns on and off on alternating magnets.
        if (Mode==1) {
            MotorDirection = false;
            if (MagnetSensed()) {
                if (MotorState) {
                    // Motor is currently on. Turn it off.
                    MotorOff();
                } else {
                    // Motor is currently off. Turn it on.
                    MotorOn(PowerLevel[Speed]);
                }
            }
        }

        // Mode 2
        //
        // In mode 2, the fan turns on at the first magnet, then turns off
        // a set time later.
        if (Mode==2) {
            MotorDirection = false;
            if (MagnetSensed()) {
                // Turn the motor on...
                MotorOn(PowerLevel[Speed]);
                // Wait a given time...
                delay(TimeLevel[Time]);
                // And turn the motor off.
                MotorOff();
            }
        }

        // Mode 3
        //
        // In mode 3, the motor turns on when it first sees a magnet, then 
        // reverses each time it passes a magnet.
        if (Mode==3) {
            if (MagnetSensed()) {
                // Reverse the motor...
                MotorDirection = !MotorDirection;
                // And turn it on in the new direction.
                MotorOn(PowerLevel[Speed]);
            }
        }

        // Add more modes here if necessary!

    }

    // Button has been pressed, so...
    MotorOff();
    MotorDirection = false;
}

void setup() {
    /* 
     * The setup function runs ONCE when power is first applied.
     *
     */

    // Set appropriate pins to output.
    pinMode(POWERINDICATOR, OUTPUT);
    pinMode(MODELIGHT, OUTPUT);
    pinMode(SPEEDLIGHT, OUTPUT);
    pinMode(TIMELIGHT, OUTPUT);
    pinMode(MOTORPWM, OUTPUT);
    pinMode(MOTORDIR, OUTPUT);

    // Set appropriate pins to input, with pull-up resistors as needed.
    pinMode(MODESWITCH, INPUT);
    digitalWrite(MODESWITCH, HIGH);     // setting an input to HIGH
    pinMode(SPEEDSWITCH, INPUT);        // turns on an internal 20k
    digitalWrite(SPEEDSWITCH, HIGH);    // pull-up resistor.
    pinMode(TIMESWITCH, INPUT);
    digitalWrite(TIMESWITCH, HIGH);
    pinMode(MAGNETSWITCH, INPUT);
    digitalWrite(MAGNETSWITCH, HIGH);   // This one may not need the 
                                        // pull-up resistor, but...
    
    // Turn off motor just in case.
    MotorOff();

    // Indicate Status.
    ShowStatus(MODELIGHT, Mode);
    ShowStatus(SPEEDLIGHT, Speed);
    ShowStatus(TIMELIGHT, Time);

    // Indicate ready-to-go
    AnnounceReady();
}

void loop() {
    /*
     * The loop() function runs repeatedly after the setup() 
     * function runs once.
     *
     */
    
    // The main loop checks for user input, passes any input to
    // the "ProgramMode" routine, then runs "DoMode". That's it!

    // Check to see whether the user wants anything
    UserInput = ButtonDown();
    CurrentTime = millis(); // Check what time it is

    while (ButtonDown()) {
        // Wait until button is up to decide what to do.
        delay(10);
    }
    if (millis()-CurrentTime > 2000) {
        // Button was held for more than 2 seconds, so program the device.
        ProgramMode(UserInput);
    } else {
        if (UserInput & B0001) {
            // The mode button was pressed.
            ShowStatus(MODELIGHT, Mode);
        } 
        if (UserInput & B0010) {
            // The speed button was pressed.
            ShowStatus(SPEEDLIGHT, Speed);
        }
        if (UserInput & B0100) {
            // The Time button was pressed.
            ShowStatus(TIMELIGHT, Time);
        }
    }
    
    // Here's where it actually does something. Whatever the mode is...
    DoMode(Mode);
}
